@@ -283,6 +283,17 @@ configureRJ <- function(conf, targetNodes, indicatorNodes = NULL, priorProb = NU
     if(indicatorFlag == priorFlag) stop("configureRJ: Provide 'indicatorNodes' or 'priorProb' vector")
     ## fixedValue can be used only with priorProb
     if(indicatorFlag && any(fixedValue != 0)) warning("configureRJ: 'fixedValue' can be provided only when using 'priorProb'; it will be ignored.")
+    ## RJ system does not support non-constant hyperparameters of target RJ nodes:
+    if(nimbleOptions('MCMCRJcheckHyperparam')) {
+        if(length(model$getParents(targetNodes, stochOnly = TRUE, includeData = FALSE)) > 0) {
+            targetNodesAsScalars <- model$expandNodeNames(targetNodes, returnScalarComponents = TRUE)
+            for(i in seq_along(targetNodesAsScalars)) {
+                if(length(model$getParents(targetNodesAsScalars[i], stochOnly = TRUE, includeData = FALSE)) > 0) {
+                    stop('Reversible jump target node \"', targetNodesAsScalars[i], '\" appears to have a non-constant hyper-parameter, which is not currently supported for reversible jump MCMC.')
+                }
+            }
+        }
+    }
     ##
     if(priorFlag) {    ## no indicator variables; use RJ_fixed_prior sampler
         ## check that priorProb values are in [0,1]
