@@ -68,7 +68,7 @@ callsFromExternalUnaries <- as.character(unlist(lapply(eigProxyTranslateExternal
 
 eigenizeCalls <- c( ## component-wise unarys valid for either Eigen array or matrix
     makeCallList(c('abs','square','sqrt','(','t'), 'eigenize_cWiseUnaryEither'),
-    makeCallList('pow', 'eigenize_cWiseByScalarArray'),
+    makeCallList(c('pow'), 'eigenize_cWiseByScalarArray'),
     makeCallList(c('asRow', 'asCol'), 'eigenize_asRowOrCol'),
     ## component-wise unarys valid only for only Eigen array
     makeCallList(c('exp','log','cube','cwiseInverse','sin','cos','tan','asin','acos'), 'eigenize_cWiseUnaryArray'), 
@@ -91,6 +91,7 @@ eigenizeCalls <- c( ## component-wise unarys valid for either Eigen array or mat
     ## matrix ops
     makeCallList(matrixSolveOperators, 'eigenize_matrixOps'),
     makeCallList('bessel_k', 'eigenize_recyclingRuleFunction'),
+    makeCallList('pow_int', 'eigenize_recyclingRuleFunction'),
     makeCallList(scalar_distribution_dFuns, 'eigenize_recyclingRuleFunction'),
     makeCallList(scalar_distribution_pFuns, 'eigenize_recyclingRuleFunction'),
     makeCallList(scalar_distribution_qFuns, 'eigenize_recyclingRuleFunction'),
@@ -611,7 +612,7 @@ eigenize_cWiseBinaryArray <- function(code, symTab, typeEnv, workEnv) {
 eigenize_cWiseByScalarArray <- function(code, symTab, typeEnv, workEnv) {
     if(code$nDim == 0) return(NULL)
     if(!is.numeric(code$args[[2]]))
-        if(code$args[[2]]$nDim != 0) stop(exprClassProcessingErrorMsg(code, 'the second argument to pow must be a scalar.'), call. = FALSE)
+        if(code$args[[2]]$nDim != 0) stop(exprClassProcessingErrorMsg(code, 'the second argument to pow or pow_int must be a scalar.'), call. = FALSE)
     newName <- eigenizeTranslate[[code$name]]
     if(is.null(newName)) stop(exprClassProcessingErrorMsg(code, 'Missing eigenizeTranslate entry.'), call. = FALSE)
     code$name <- newName
