@@ -91,6 +91,7 @@ toEigenCalls <- c(
                    'ISNA'), 'toEigenScalarRecurse'),
     makeCallList(c('nimArr_dmnorm_chol',
                    'nimArr_dmvt_chol',
+                   'nimArr_dlkj_corr_cholesky',
                    'nimArr_dwish_chol',
                    'nimArr_dinvwish_chol',
                    'nimArr_dmulti',
@@ -99,6 +100,7 @@ toEigenCalls <- c(
                    'nimArr_ddirch'), 'toEigenScalarRecurse'),
     makeCallList(c('nimArr_rmnorm_chol',
                    'nimArr_rmvt_chol',
+                   'nimArr_rlkj_corr_cholesky',
                    'nimArr_rwish_chol',
                    'nimArr_rinvwish_chol',
                    'nimArr_rmulti',
